@@ -26,7 +26,7 @@
 #include <xorg-config.h>
 #endif
 
-#ifdef i386
+#if defined(__i386__) || defined(__i386)
 #define _NEED_SYSI86
 #endif
 #include "xf86.h"
@@ -62,11 +62,11 @@ xf86ReadBIOS(unsigned long Base, unsigned long Offset, unsigned char *Buf,
 	 *
 	 *	Use /dev/xsvc for everything.
 	 */
-	psize = xf86getpagesize();
+	psize = getpagesize();
 	Offset += Base & (psize - 1);
 	Base &= ~(psize - 1);
 	mlen = (Offset + Len + psize - 1) & ~(psize - 1);
-#if defined(i386) && !defined(__SOL8__)
+#if (defined(__i386__) || defined(__i386)) && !defined(__SOL8__)
 	if (Base >= 0xA0000 && Base + mlen < 0xFFFFF && xf86Info.vtno >= 0)
 		sprintf(solx86_vtname, "/dev/vt%02d", xf86Info.vtno);
 	else
